// ------------------------------------------------------------------------------
//  <autogenerated>
//      This code was generated by a tool.
//      Mono Runtime Version: 4.0.30319.1
// 
//      Changes to this file may cause incorrect behavior and will be lost if 
//      the code is regenerated.
//  </autogenerated>
// ------------------------------------------------------------------------------

namespace SparkleShare {
	
	
	// Should subclass AppKit.NSResponder
	[Foundation.Register("AppDelegate")]
	public partial class AppDelegate {
	}
}
