<Window x:Class="SparkleShare.Note"
             xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
             xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
             xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" 
             xmlns:d="http://schemas.microsoft.com/expression/blend/2008" 
             mc:Ignorable="d" 
             d:DesignHeight="240" d:DesignWidth="480" ResizeMode="NoResize" WindowStartupLocation="CenterScreen" Width="480" Height="240">
    <Grid x:Name="cover" Margin="18">
        <Grid.RowDefinitions>
            <RowDefinition Height="60" />
            <RowDefinition Height="72" />
            <RowDefinition Height="*" />
        </Grid.RowDefinitions>
        <Grid.ColumnDefinitions>
            <ColumnDefinition Width="48" />
            <ColumnDefinition Width="*" />
        </Grid.ColumnDefinitions>
        <Rectangle RadiusX="5" RadiusY="5" Width="48" Height="48" StrokeThickness="0" >
            <Rectangle.Fill>
                <ImageBrush  x:Name="user_image" />
            </Rectangle.Fill>
        </Rectangle>
        <StackPanel Grid.Row="0" Grid.Column="1" Margin="18,0,0,0" VerticalAlignment="Center" HorizontalAlignment="left" Orientation="Vertical">
            <TextBlock x:Name="user_name_text_block" TextAlignment="Left" Height="22" Width="320" Grid.Column="1" FontWeight="Bold"></TextBlock>
            <TextBlock x:Name="user_email_text_field" TextAlignment="Left" Height="20" Width="320"  Grid.Column="1"></TextBlock>
        </StackPanel>
        <TextBox x:Name="balloon_text_field" Grid.Row="1" Grid.ColumnSpan="2" Width="438" Height="72" BorderBrush="{x:Null}" BorderThickness="0" Padding="8,12,8,8" TextWrapping="Wrap" AcceptsReturn="True" Text="Anything to add?" >
            <TextBox.Background>
                <ImageBrush ImageSource="pack://application:,,,/SparkleShare.Windows;component/Images/text-balloon.png" Stretch="Uniform"></ImageBrush>
            </TextBox.Background>
        </TextBox>
        <StackPanel Grid.Column="1" Grid.Row="2" VerticalAlignment="Bottom" HorizontalAlignment="Right" Orientation="Horizontal">
            <Button x:Name="cancel_button" Content="Cancel" Width="75" Margin="0,0,9,0" VerticalAlignment="Center" IsCancel="True"></Button>
            <Button x:Name="sync_button" Content="Sync"  Width="75" VerticalAlignment="Center" IsDefault="True"/>
        </StackPanel>
    </Grid>
</Window>
